@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -38,7 +38,7 @@ def getLoadedPackages():
     """
 
     for module_name, module in sys.modules.items():
-        if not hasattr(module, "__path__"):
+        if not getattr(module, "__path__", None):
             continue
 
         if hasattr(module, "__file__"):
@@ -93,13 +93,33 @@ def isPreloadedPackagePath(path):
     return False
 
 
+def _considerPthImportedPackage(module_name):
+    if module_name in ("os", "sys"):
+        return None
+
+    if module_name.startswith("__editable__"):
+        finder_module = __import__(module_name)
+
+        paths = set()
+
+        mapping = getattr(finder_module, "MAPPING", {})
+        for package_name, path in mapping.items():
+            if os.path.basename(path) != package_name:
+                continue
+
+            paths.add(os.path.dirname(path))
+
+        sys.path.extend(sorted(paths))
+
+        return None
+
+    return module_name
+
+
 def detectPthImportedPackages():
     if not hasattr(sys.modules["site"], "getsitepackages"):
         return ()
 
-    # TODO: Move hard import config to elsewhere.
-    from nuitka.nodes.ImportNodes import isHardModuleWithoutSideEffect
-
     pth_imports = set()
 
     for prefix in sys.modules["site"].getsitepackages():
@@ -115,9 +135,9 @@ def detectPthImportedPackages():
                                 line = line[: line.find(";")]
 
                             for part in line[7:].split(","):
-                                pth_import = part.strip()
+                                pth_import = _considerPthImportedPackage(part.strip())
 
-                                if not isHardModuleWithoutSideEffect(pth_import):
+                                if pth_import is not None:
                                     pth_imports.add(pth_import)
                 except OSError:
                     recursion_logger.warning(
