@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,66 +22,50 @@
 from nuitka.specs.BuiltinParameterSpecs import builtin_dict_spec
 
 from .BuiltinIteratorNodes import ExpressionBuiltinIter1
-from .ConstantRefNodes import makeConstantRefNode
-from .DictionaryNodes import ExpressionKeyValuePair, ExpressionMakeDict
-from .ExpressionBases import ExpressionChildrenHavingBase
+from .ChildrenHavingMixins import ChildrenHavingPosArgOptionalPairsTupleMixin
+from .DictionaryNodes import makeExpressionMakeDict
+from .ExpressionBases import ExpressionBase
+from .ExpressionShapeMixins import ExpressionDictShapeExactMixin
 from .NodeMakingHelpers import wrapExpressionWithNodeSideEffects
 
 
-class ExpressionBuiltinDict(ExpressionChildrenHavingBase):
+class ExpressionBuiltinDict(
+    ExpressionDictShapeExactMixin,
+    ChildrenHavingPosArgOptionalPairsTupleMixin,
+    ExpressionBase,
+):
     kind = "EXPRESSION_BUILTIN_DICT"
 
-    named_children = (
-        "pos_arg",
-        "pairs"
-    )
+    named_children = ("pos_arg|optional", "pairs|tuple")
 
     def __init__(self, pos_arg, pairs, source_ref):
-        assert type(pos_arg) not in (tuple, list), source_ref
-        assert type(pairs) in (tuple, list), source_ref
-
-        ExpressionChildrenHavingBase.__init__(
+        ChildrenHavingPosArgOptionalPairsTupleMixin.__init__(
             self,
-            values     = {
-                "pos_arg" : pos_arg,
-                "pairs"   : tuple(
-                    ExpressionKeyValuePair(
-                        makeConstantRefNode(key, source_ref),
-                        value,
-                        value.getSourceReference()
-                    )
-                    for key, value in
-                    pairs
-                )
-            },
-            source_ref = source_ref
+            pos_arg=pos_arg,
+            pairs=pairs,
         )
 
-    getPositionalArgument = ExpressionChildrenHavingBase.childGetter("pos_arg")
-    getNamedArgumentPairs = ExpressionChildrenHavingBase.childGetter("pairs")
+        ExpressionBase.__init__(self, source_ref)
 
     def hasOnlyConstantArguments(self):
-        pos_arg = self.getPositionalArgument()
+        pos_arg = self.subnode_pos_arg
 
         if pos_arg is not None and not pos_arg.isCompileTimeConstant():
             return False
 
-        for arg_pair in self.getNamedArgumentPairs():
-            if not arg_pair.getKey().isCompileTimeConstant():
-                return False
-            if not arg_pair.getValue().isCompileTimeConstant():
+        for arg_pair in self.subnode_pairs:
+            if not arg_pair.isCompileTimeConstant():
                 return False
 
         return True
 
     def computeExpression(self, trace_collection):
-        pos_arg = self.getPositionalArgument()
-        pairs = self.getNamedArgumentPairs()
+        pos_arg = self.subnode_pos_arg
+        pairs = self.subnode_pairs
 
         if pos_arg is None:
-            new_node = ExpressionMakeDict(
-                pairs      = self.getNamedArgumentPairs(),
-                source_ref = self.source_ref
+            new_node = makeExpressionMakeDict(
+                pairs=self.subnode_pairs, source_ref=self.source_ref
             )
 
             # This cannot raise anymore than its arguments, as the keys will
@@ -90,24 +74,22 @@ class ExpressionBuiltinDict(ExpressionChildrenHavingBase):
             return (
                 new_node,
                 "new_expression",
-                "Replace 'dict' built-in call dictionary creation from arguments."
+                "Replace 'dict' built-in call dictionary creation from arguments.",
             )
 
         pos_iteration_length = pos_arg.getIterationLength()
 
         if pos_iteration_length == 0:
-            new_node = ExpressionMakeDict(
-                pairs      = self.getNamedArgumentPairs(),
-                source_ref = self.source_ref
+            new_node = makeExpressionMakeDict(
+                pairs=self.subnode_pairs, source_ref=self.source_ref
             )
 
             # Maintain potential side effects from the positional arguments.
             new_node = wrapExpressionWithNodeSideEffects(
-                old_node = ExpressionBuiltinIter1(
-                    value      = pos_arg,
-                    source_ref = self.source_ref
+                old_node=ExpressionBuiltinIter1(
+                    value=pos_arg, source_ref=self.source_ref
                 ),
-                new_node = new_node
+                new_node=new_node,
             )
 
             # Just in case, the iteration may do that.
@@ -117,25 +99,25 @@ class ExpressionBuiltinDict(ExpressionChildrenHavingBase):
             return (
                 new_node,
                 "new_expression",
-                "Replace 'dict' built-in call dictionary creation from arguments."
+                "Replace 'dict' built-in call dictionary creation from arguments.",
             )
 
-        if pos_iteration_length is not None and \
-           pos_iteration_length + len(pairs) < 256 and \
-           self.hasOnlyConstantArguments():
+        if (
+            pos_iteration_length is not None
+            and pos_iteration_length + len(pairs) < 256
+            and self.hasOnlyConstantArguments()
+        ):
             if pos_arg is not None:
-                pos_args = (
-                    pos_arg,
-                )
+                pos_args = (pos_arg,)
             else:
                 pos_args = None
 
             return trace_collection.getCompileTimeComputationResult(
-                node        = self,
-                computation = lambda : builtin_dict_spec.simulateCall(
-                    (pos_args, self.getNamedArgumentPairs())
+                node=self,
+                computation=lambda: builtin_dict_spec.simulateCall(
+                    (pos_args, self.subnode_pairs)
                 ),
-                description = "Replace 'dict' call with constant arguments."
+                description="Replace 'dict' call with constant arguments.",
             )
         else:
             trace_collection.onExceptionRaiseExit(BaseException)
@@ -143,17 +125,14 @@ class ExpressionBuiltinDict(ExpressionChildrenHavingBase):
             return self, None, None
 
     def mayRaiseException(self, exception_type):
-        pos_arg = self.getPositionalArgument()
+        pos_arg = self.subnode_pos_arg
 
         # TODO: Determining if it's sufficient is not easy but possible.
         if pos_arg is not None:
             return True
 
-        for arg_pair in self.getNamedArgumentPairs():
+        for arg_pair in self.subnode_pairs:
             if arg_pair.mayRaiseException(exception_type):
                 return True
 
         return False
-
-    def hasShapeDictionaryExact(self):
-        return True
