@@ -98,7 +98,7 @@ class ExpressionConstantUntrackedRefBase(CompileTimeConstantExpressionBase):
     __slots__ = ("constant",)
 
     def __init__(self, constant, source_ref):
-        CompileTimeConstantExpressionBase.__init__(self, source_ref=source_ref)
+        CompileTimeConstantExpressionBase.__init__(self, source_ref)
 
         self.constant = constant
 
@@ -377,7 +377,7 @@ class ExpressionConstantBoolRefBase(
     @staticmethod
     def computeExpressionBool(trace_collection):
         # Best case already, None indicated no action.
-        return None, None
+        return None, None, None
 
     @staticmethod
     def getDetails():
