@@ -30,7 +30,7 @@ from abc import abstractmethod
 
 from nuitka import Options, Tracing, TreeXML, Variables
 from nuitka.__past__ import iterItems
-from nuitka.Errors import NuitkaNodeDesignError, NuitkaNodeError
+from nuitka.Errors import NuitkaNodeError
 from nuitka.PythonVersions import python_version
 from nuitka.SourceCodeReferences import SourceCodeReference
 from nuitka.utils.InstanceCounters import (
@@ -169,12 +169,8 @@ class NodeBase(NodeMetaClassBase):
     def getChildNameNice(self):
         child_name = self.getChildName()
 
-        if hasattr(self.parent, "nice_children"):
-            return self.parent.nice_children[
-                self.parent.named_children.index(child_name)
-            ]
-        elif hasattr(self.parent, "nice_child"):
-            return self.parent.nice_child
+        if hasattr(self.parent, "nice_children_dict"):
+            return self.parent.nice_children_dict[child_name]
         else:
             return child_name
 
@@ -323,6 +319,10 @@ class NodeBase(NodeMetaClassBase):
 
         Tracing.printSeparator(level)
 
+    @staticmethod
+    def isStatementsSequence():
+        return False
+
     @staticmethod
     def isStatementsFrame():
         return False
@@ -332,11 +332,13 @@ class NodeBase(NodeMetaClassBase):
         # For overload by module nodes
         return False
 
+    @abstractmethod
     def isExpression(self):
-        return self.kind.startswith("EXPRESSION_")
+        """Is an expression node."""
 
+    @abstractmethod
     def isStatement(self):
-        return self.kind.startswith("STATEMENT_")
+        """Is a statement node."""
 
     def isExpressionBuiltin(self):
         return self.kind.startswith("EXPRESSION_BUILTIN_")
@@ -349,6 +351,10 @@ class NodeBase(NodeMetaClassBase):
     def isStatementDelVariable():
         return False
 
+    @staticmethod
+    def isStatementReleaseVariable():
+        return False
+
     @staticmethod
     def isExpressionConstantRef():
         return False
@@ -401,6 +407,10 @@ class NodeBase(NodeMetaClassBase):
     def isExpressionImportModuleNameHard():
         return False
 
+    @staticmethod
+    def isExpressionFunctionCreation():
+        return False
+
     def visit(self, context, visitor):
         visitor(self)
 
@@ -467,9 +477,7 @@ class NodeBase(NodeMetaClassBase):
         return self.mayRaiseException(BaseException)
 
     @staticmethod
-    def willRaiseException(exception_type):
-        """Unless we are told otherwise, nothing may raise anything."""
-        # Virtual method, pylint: disable=unused-argument
+    def willRaiseAnyException():
         return False
 
     @staticmethod
@@ -479,7 +487,7 @@ class NodeBase(NodeMetaClassBase):
 
 
 class CodeNodeMixin(object):
-    # Mixins are not allow to specify slots, pylint: disable=assigning-non-slot
+    # Mixins are not allowed to specify slots, pylint: disable=assigning-non-slot
     __slots__ = ()
 
     def __init__(self, name, code_prefix):
@@ -527,202 +535,10 @@ class CodeNodeMixin(object):
         return self.uids[node.kind]
 
 
-class ChildrenHavingMixin(object):
-    # Mixins are not allow to specify slots.
-    __slots__ = ()
-
-    named_children = ()
-
-    checkers = {}
-
-    def __init__(self, values):
-        assert (
-            type(self.named_children) is tuple and self.named_children
-        ), self.named_children
-
-        # TODO: Make this true.
-        # assert len(self.named_children) > 1, self.kind
-
-        # Check for completeness of given values, everything should be there
-        # but of course, might be put to None.
-        if set(values.keys()) != set(self.named_children):
-            raise NuitkaNodeDesignError(
-                "Must pass named children in value dictionary",
-                set(values.keys()),
-                set(self.named_children),
-            )
-
-        for name, value in values.items():
-            if name in self.checkers:
-                value = self.checkers[name](value)
-
-            if type(value) is tuple:
-                assert None not in value, name
-
-                for val in value:
-                    val.parent = self
-            elif value is None:
-                pass
-            else:
-                value.parent = self
-
-            attr_name = "subnode_" + name
-            setattr(self, attr_name, value)
-
-    def setChild(self, name, value):
-        """Set a child value.
-
-        Do not overload, provider self.checkers instead.
-        """
-        # Only accept legal child names
-        assert name in self.named_children, name
-
-        # Lists as inputs are OK, but turn them into tuples.
-        if type(value) is list:
-            value = tuple(value)
-
-        if name in self.checkers:
-            value = self.checkers[name](value)
-
-        # Re-parent value to us.
-        if type(value) is tuple:
-            for val in value:
-                val.parent = self
-        elif value is not None:
-            value.parent = self
-
-        attr_name = "subnode_" + name
-
-        # Determine old value, and inform it about losing its parent.
-        old_value = getattr(self, attr_name)
-
-        assert old_value is not value, value
-
-        setattr(self, attr_name, value)
-
-    def clearChild(self, name):
-        # Only accept legal child names
-        assert name in self.named_children, name
-
-        if name in self.checkers:
-            self.checkers[name](None)
-
-        attr_name = "subnode_" + name
-
-        # Determine old value, and inform it about losing its parent.
-        old_value = getattr(self, attr_name)
-
-        assert old_value is not None
-
-        setattr(self, attr_name, None)
-
-    def getChild(self, name):
-        attr_name = "subnode_" + name
-        return getattr(self, attr_name)
-
-    def getVisitableNodes(self):
-        # TODO: Consider if a generator would be faster.
-        result = []
-
-        for name in self.named_children:
-            attr_name = "subnode_" + name
-
-            value = getattr(self, attr_name)
-
-            if value is None:
-                pass
-            elif type(value) is tuple:
-                result += list(value)
-            elif isinstance(value, NodeBase):
-                result.append(value)
-            else:
-                raise AssertionError(
-                    self, "has illegal child", name, value, value.__class__
-                )
-
-        return tuple(result)
-
-    def getVisitableNodesNamed(self):
-        """Named children dictionary.
-
-        For use in debugging and XML output.
-        """
-        for name in self.named_children:
-            attr_name = "subnode_" + name
-            value = getattr(self, attr_name)
-
-            yield name, value
-
-    def replaceChild(self, old_node, new_node):
-        if new_node is not None and not isinstance(new_node, NodeBase):
-            raise AssertionError(
-                "Cannot replace with", new_node, "old", old_node, "in", self
-            )
-
-        # Find the replaced node, as an added difficulty, what might be
-        # happening, is that the old node is an element of a tuple, in which we
-        # may also remove that element, by setting it to None.
-        for key in self.named_children:
-            value = self.getChild(key)
-
-            if value is None:
-                pass
-            elif type(value) is tuple:
-                if old_node in value:
-                    if new_node is not None:
-                        self.setChild(
-                            key,
-                            tuple(
-                                (val if val is not old_node else new_node)
-                                for val in value
-                            ),
-                        )
-                    else:
-                        self.setChild(
-                            key, tuple(val for val in value if val is not old_node)
-                        )
-
-                    return key
-            elif isinstance(value, NodeBase):
-                if old_node is value:
-                    self.setChild(key, new_node)
-
-                    return key
-            else:
-                assert False, (key, value, value.__class__)
-
-        raise AssertionError("Didn't find child", old_node, "in", self)
-
-    def getCloneArgs(self):
-        values = {}
-
-        for key in self.named_children:
-            value = self.getChild(key)
-
-            assert type(value) is not list, key
-
-            if value is None:
-                values[key] = None
-            elif type(value) is tuple:
-                values[key] = tuple(v.makeClone() for v in value)
-            else:
-                values[key] = value.makeClone()
-
-        values.update(self.getDetails())
-
-        return values
-
-    def finalize(self):
-        del self.parent
-
-        for c in self.getVisitableNodes():
-            c.finalize()
-
-
 class ClosureGiverNodeMixin(CodeNodeMixin):
     """Base class for nodes that provide variables for closure takers."""
 
-    # Mixins are not allow to specify slots, pylint: disable=assigning-non-slot
+    # Mixins are not allowed to specify slots, pylint: disable=assigning-non-slot
     __slots__ = ()
 
     def __init__(self, name, code_prefix):
@@ -826,7 +642,7 @@ class ClosureGiverNodeMixin(CodeNodeMixin):
 class ClosureTakerMixin(object):
     """Mixin for nodes that accept variables from closure givers."""
 
-    # Mixins are not allow to specify slots, pylint: disable=assigning-non-slot
+    # Mixins are not allowed to specify slots, pylint: disable=assigning-non-slot
     __slots__ = ()
 
     def __init__(self, provider):
@@ -884,15 +700,23 @@ class ClosureTakerMixin(object):
 
 
 class StatementBase(NodeBase):
-    """Base class for all statements."""
+    """Base class for all statement nodes."""
 
     # Base classes can be abstract, pylint: disable=abstract-method
 
+    @staticmethod
+    def isStatement():
+        return True
+
+    @staticmethod
+    def isExpression():
+        return False
+
     # TODO: Have them all.
     # @abstractmethod
     @staticmethod
     def getStatementNiceName():
-        return "undescribed statement"
+        return "un-described statement"
 
     def computeStatementSubExpressions(self, trace_collection):
         """Compute a statement.
@@ -906,9 +730,9 @@ class StatementBase(NodeBase):
         for count, expression in enumerate(expressions):
             assert expression.isExpression(), (self, expression)
 
-            expression = trace_collection.onExpression(expression=expression)
+            expression = trace_collection.onExpression(expression)
 
-            if expression.willRaiseException(BaseException):
+            if expression.willRaiseAnyException():
                 wrapped_expression = makeStatementOnlyNodesFromExpressions(
                     expressions[: count + 1]
                 )
@@ -925,172 +749,8 @@ class StatementBase(NodeBase):
         return self, None, None
 
 
-class StatementChildrenHavingBase(ChildrenHavingMixin, StatementBase):
-    def __init__(self, values, source_ref):
-        StatementBase.__init__(self, source_ref=source_ref)
-
-        ChildrenHavingMixin.__init__(self, values=values)
-
-
-class StatementChildHavingBase(StatementBase):
-    named_child = ""
-
-    checker = None
-
-    def __init__(self, value, source_ref):
-        StatementBase.__init__(self, source_ref=source_ref)
-
-        if self.checker is not None:
-            value = self.checker(value)  # False alarm, pylint: disable=not-callable
-
-        assert type(value) is not list, self.named_child
-
-        if type(value) is tuple:
-            assert None not in value, self.named_child
-
-            for val in value:
-                val.parent = self
-        elif value is not None:
-            value.parent = self
-        elif value is None:
-            pass
-        else:
-            assert False, type(value)
-
-        attr_name = "subnode_" + self.named_child
-        setattr(self, attr_name, value)
-
-    def setChild(self, name, value):
-        """Set a child value.
-
-        Do not overload, provider self.checkers instead.
-        """
-        # Only accept legal child names
-        assert name == self.named_child, name
-
-        # Lists as inputs are OK, but turn them into tuples.
-        if type(value) is list:
-            value = tuple(value)
-
-        if self.checker is not None:
-            value = self.checker(value)  # False alarm, pylint: disable=not-callable
-
-        # Re-parent value to us.
-        if type(value) is tuple:
-            for val in value:
-                val.parent = self
-        elif value is not None:
-            value.parent = self
-
-        attr_name = "subnode_" + name
-
-        # Determine old value, and inform it about losing its parent.
-        old_value = getattr(self, attr_name)
-
-        assert old_value is not value, value
-
-        setattr(self, attr_name, value)
-
-    def getChild(self, name):
-        # Only accept legal child names
-        attr_name = "subnode_" + name
-        return getattr(self, attr_name)
-
-    def getVisitableNodes(self):
-        # TODO: Consider if a generator would be faster.
-        attr_name = "subnode_" + self.named_child
-        value = getattr(self, attr_name)
-
-        if value is None:
-            return ()
-        elif type(value) is tuple:
-            return value
-        elif isinstance(value, NodeBase):
-            return (value,)
-        else:
-            raise AssertionError(self, "has illegal child", value, value.__class__)
-
-    def getVisitableNodesNamed(self):
-        """Named children dictionary.
-
-        For use in debugging and XML output.
-        """
-        attr_name = "subnode_" + self.named_child
-        value = getattr(self, attr_name)
-
-        yield self.named_child, value
-
-    def replaceChild(self, old_node, new_node):
-        if new_node is not None and not isinstance(new_node, NodeBase):
-            raise AssertionError(
-                "Cannot replace with", new_node, "old", old_node, "in", self
-            )
-
-        # Find the replaced node, as an added difficulty, what might be
-        # happening, is that the old node is an element of a tuple, in which we
-        # may also remove that element, by setting it to None.
-        key = self.named_child
-        value = self.getChild(key)
-
-        if value is None:
-            pass
-        elif type(value) is tuple:
-            if old_node in value:
-                if new_node is not None:
-                    self.setChild(
-                        key,
-                        tuple(
-                            (val if val is not old_node else new_node) for val in value
-                        ),
-                    )
-                else:
-                    self.setChild(
-                        key, tuple(val for val in value if val is not old_node)
-                    )
-
-                return key
-        elif isinstance(value, NodeBase):
-            if old_node is value:
-                self.setChild(key, new_node)
-
-                return key
-        else:
-            assert False, (key, value, value.__class__)
-
-        raise AssertionError("Didn't find child", old_node, "in", self)
-
-    def getCloneArgs(self):
-        # Make clones of child nodes too.
-        values = {}
-        key = self.named_child
-
-        value = self.getChild(key)
-
-        assert type(value) is not list, key
-
-        if value is None:
-            values[key] = None
-        elif type(value) is tuple:
-            values[key] = tuple(v.makeClone() for v in value)
-        else:
-            values[key] = value.makeClone()
-
-        values.update(self.getDetails())
-
-        return values
-
-    def finalize(self):
-        del self.parent
-
-        attr_name = "subnode_" + self.named_child
-        child = getattr(self, attr_name)
-        if child is not None:
-            child.finalize()
-        delattr(self, attr_name)
-
-
 class SideEffectsFromChildrenMixin(object):
-    # Mixins are not allow to specify slots.
+    # Mixins are not allowed to specify slots.
     __slots__ = ()
 
     def mayHaveSideEffects(self):
