@@ -26,6 +26,7 @@ make others possible.
 import inspect
 
 from nuitka import ModuleRegistry, Options, Variables
+from nuitka.importing.Importing import addExtraSysPaths
 from nuitka.importing.Recursion import considerUsedModules
 from nuitka.plugins.Plugins import Plugins
 from nuitka.Progress import (
@@ -181,6 +182,8 @@ def optimizeModule(module):
     global tag_set
     tag_set = TagSet()
 
+    addExtraSysPaths(Plugins.getModuleSysPathAdditions(module.getFullName()))
+
     if module.isPythonExtensionModule():
         optimizeExtensionModule(module)
         changed = False
@@ -324,14 +327,12 @@ def makeOptimizationPass():
                 unused_function.trace_collection = None
                 unused_function.finalize()
 
-            used_functions = tuple(
+            current_module.subnode_functions = tuple(
                 function
                 for function in current_module.subnode_functions
                 if function in current_module.getUsedFunctions()
             )
 
-            current_module.setChild("functions", used_functions)
-
     _endProgress()
 
     return finished
