@@ -263,8 +263,8 @@ class NuitkaPluginImplicitImports(NuitkaPluginBase):
         # TODO: This code absolutely doesn't belong here.
         # Read the .pyi file, and provide as implicit dependency.
         if module.isPythonExtensionModule():
-            for used_module in module.getUsedModules():
-                yield used_module[0]
+            for used_module_name in module.getPyIModuleImportedNames():
+                yield used_module_name
 
         if full_name == "pkg_resources.extern":
             # TODO: A package specific lookup of compile time "pkg_resources.extern" could
@@ -279,6 +279,7 @@ class NuitkaPluginImplicitImports(NuitkaPluginBase):
                 "importlib_resources",
                 "more_itertools",
                 "six",
+                "platformdirs",
             ):
                 yield "pkg_resources._vendor." + part
 
@@ -321,6 +322,18 @@ class NuitkaPluginImplicitImports(NuitkaPluginBase):
                 else:
                     yield module_filename
 
+    def _getModuleSysPathAdditions(self, module_name, config):
+        module_filename = self.locateModule(module_name)
+
+        if os.path.isfile(module_filename):
+            module_filename = yield os.path.dirname(module_filename)
+
+        for relative_path in config.get("global-sys-path", ()):
+            candidate = os.path.abspath(os.path.join(module_filename, relative_path))
+
+            if os.path.isdir(candidate):
+                yield candidate
+
     def getModuleSpecificDllPaths(self, module_name):
         for entry in self.config.get(module_name, section="import-hacks"):
             if self.evaluateCondition(
@@ -329,6 +342,16 @@ class NuitkaPluginImplicitImports(NuitkaPluginBase):
                 for item in self._getModuleSpecificDllPaths(config=entry):
                     yield item
 
+    def getModuleSysPathAdditions(self, module_name):
+        for entry in self.config.get(module_name, section="import-hacks"):
+            if self.evaluateCondition(
+                full_name=module_name, condition=entry.get("when", "True")
+            ):
+                for item in self._getModuleSysPathAdditions(
+                    module_name=module_name, config=entry
+                ):
+                    yield item
+
     def onModuleSourceCode(self, module_name, source_code):
         if module_name == "numexpr.cpuinfo":
 
