@@ -42,11 +42,11 @@ from nuitka.nodes.ConstantRefNodes import (
     makeConstantRefNode,
 )
 from nuitka.nodes.ContainerMakingNodes import makeExpressionMakeTupleOrConstant
-from nuitka.nodes.ContainerOperationNodes import ExpressionListOperationPop
 from nuitka.nodes.InjectCNodes import (
     StatementInjectCCode,
     StatementInjectCDecl,
 )
+from nuitka.nodes.ListOperationNodes import ExpressionListOperationPop1
 from nuitka.nodes.NodeMakingHelpers import (
     makeRaiseExceptionExpressionFromTemplate,
 )
@@ -68,10 +68,7 @@ from nuitka.nodes.SubscriptNodes import (
     StatementDelSubscript,
 )
 from nuitka.nodes.VariableAssignNodes import makeStatementAssignmentVariable
-from nuitka.nodes.VariableDelNodes import (
-    StatementReleaseVariable,
-    makeStatementDelVariable,
-)
+from nuitka.nodes.VariableDelNodes import makeStatementDelVariable
 from nuitka.nodes.VariableNameNodes import (
     ExpressionVariableLocalNameRef,
     ExpressionVariableNameRef,
@@ -79,6 +76,7 @@ from nuitka.nodes.VariableNameNodes import (
     StatementDelVariableName,
 )
 from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
+from nuitka.nodes.VariableReleaseNodes import makeStatementReleaseVariable
 from nuitka.Options import hasPythonFlagNoAnnotations, isExperimental
 from nuitka.PythonVersions import python_version
 from nuitka.Tracing import general
@@ -124,7 +122,7 @@ def buildExtSliceNode(provider, node, source_ref):
         elements.append(element)
 
     return makeExpressionMakeTupleOrConstant(
-        elements=elements, user_provided=True, source_ref=source_ref
+        elements=tuple(elements), user_provided=True, source_ref=source_ref
     )
 
 
@@ -182,9 +180,9 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source, source_
         # For Python3 there is no slicing operation, this is always done
         # with subscript using a slice object. For Python2, it is only done
         # if no "step" is provided.
-        use_sliceobj = python_version >= 0x300
+        use_slice_object = python_version >= 0x300
 
-        if use_sliceobj:
+        if use_slice_object:
             return StatementAssignmentSubscript(
                 subscribed=lookup_source,
                 source=source,
@@ -241,7 +239,7 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source, source_
                     starred_index + 1,
                     makeStatementAssignmentVariable(
                         variable=element_var,
-                        source=ExpressionListOperationPop(
+                        source=ExpressionListOperationPop1(
                             list_arg=ExpressionTempVariableRef(
                                 variable=starred_list_var, source_ref=source_ref
                             ),
@@ -351,7 +349,7 @@ not enough values to unpack (expected at least %d, got %%d)"""
                 provider=provider,
                 tried=statements,
                 final=(
-                    StatementReleaseVariable(
+                    makeStatementReleaseVariable(
                         variable=source_iter_var, source_ref=source_ref
                     ),
                 ),
@@ -391,7 +389,9 @@ not enough values to unpack (expected at least %d, got %%d)"""
 
         for element_var in element_vars:
             final_statements.append(
-                StatementReleaseVariable(variable=element_var, source_ref=source_ref)
+                makeStatementReleaseVariable(
+                    variable=element_var, source_ref=source_ref
+                )
             )
 
         return makeTryFinallyStatement(
@@ -583,7 +583,9 @@ def buildAssignNode(provider, node, source_ref):
         return makeTryFinallyStatement(
             provider=provider,
             tried=statements,
-            final=StatementReleaseVariable(variable=tmp_source, source_ref=source_ref),
+            final=makeStatementReleaseVariable(
+                variable=tmp_source, source_ref=source_ref
+            ),
             source_ref=source_ref,
         )
 
@@ -698,9 +700,9 @@ def buildDeleteStatementFromDecoded(provider, kind, detail, source_ref):
     elif kind == "Slice":
         lookup_source, lower, upper = detail
 
-        use_sliceobj = python_version >= 0x300
+        use_slice_object = python_version >= 0x300
 
-        if use_sliceobj:
+        if use_slice_object:
             return StatementDelSubscript(
                 subscribed=lookup_source,
                 subscript=makeExpressionBuiltinSlice(
@@ -834,7 +836,7 @@ def _buildInplaceAssignAttributeNode(
         makeTryFinallyStatement(
             provider=provider,
             tried=(inplace_to_tmp, copy_back_from_tmp),
-            final=StatementReleaseVariable(
+            final=makeStatementReleaseVariable(
                 variable=tmp_variable, source_ref=source_ref
             ),
             source_ref=source_ref,
@@ -907,9 +909,15 @@ def _buildInplaceAssignSubscriptNode(
             provider=provider,
             tried=statements,
             final=(
-                StatementReleaseVariable(variable=tmp_variable1, source_ref=source_ref),
-                StatementReleaseVariable(variable=tmp_variable2, source_ref=source_ref),
-                StatementReleaseVariable(variable=tmp_variable3, source_ref=source_ref),
+                makeStatementReleaseVariable(
+                    variable=tmp_variable1, source_ref=source_ref
+                ),
+                makeStatementReleaseVariable(
+                    variable=tmp_variable2, source_ref=source_ref
+                ),
+                makeStatementReleaseVariable(
+                    variable=tmp_variable3, source_ref=source_ref
+                ),
             ),
             source_ref=source_ref,
         ),
@@ -940,7 +948,7 @@ def _buildInplaceAssignSliceNode(
     )
 
     final_statements = [
-        StatementReleaseVariable(variable=tmp_variable1, source_ref=source_ref)
+        makeStatementReleaseVariable(variable=tmp_variable1, source_ref=source_ref)
     ]
     statements = []
 
@@ -951,7 +959,7 @@ def _buildInplaceAssignSliceNode(
             )
         )
         final_statements.append(
-            StatementReleaseVariable(variable=tmp_variable2, source_ref=source_ref)
+            makeStatementReleaseVariable(variable=tmp_variable2, source_ref=source_ref)
         )
 
         lower_ref1 = ExpressionTempVariableRef(
@@ -972,7 +980,7 @@ def _buildInplaceAssignSliceNode(
             )
         )
         final_statements.append(
-            StatementReleaseVariable(variable=tmp_variable3, source_ref=source_ref)
+            makeStatementReleaseVariable(variable=tmp_variable3, source_ref=source_ref)
         )
 
         upper_ref1 = ExpressionTempVariableRef(
@@ -986,10 +994,10 @@ def _buildInplaceAssignSliceNode(
 
         upper_ref1 = upper_ref2 = None
 
-    use_sliceobj = python_version >= 0x300
+    use_slice_object = python_version >= 0x300
 
     # Second assign the in-place result over the original value.
-    if use_sliceobj:
+    if use_slice_object:
         statements += (
             makeStatementAssignmentVariable(
                 variable=tmp_variable4,
@@ -1072,7 +1080,7 @@ def _buildInplaceAssignSliceNode(
         )
 
     final_statements.append(
-        StatementReleaseVariable(variable=tmp_variable4, source_ref=source_ref)
+        makeStatementReleaseVariable(variable=tmp_variable4, source_ref=source_ref)
     )
 
     return (
@@ -1236,18 +1244,17 @@ def buildNamedExprNode(provider, node, source_ref):
         ),
     )
 
-    outline_body.setChild(
-        "body",
+    outline_body.setChildBody(
         makeStatementsSequenceFromStatement(
             statement=makeTryFinallyStatement(
                 provider=provider,
                 tried=statements,
-                final=StatementReleaseVariable(
+                final=makeStatementReleaseVariable(
                     variable=tmp_value, source_ref=source_ref
                 ),
                 source_ref=source_ref,
             )
-        ),
+        )
     )
 
     return outline_body
