@@ -196,14 +196,14 @@ def _buildMatchSequence(provider, pattern, make_against, source_ref):
             offset = -(len(pattern.patterns) - count)
 
         if seq_pattern.__class__ is ast.MatchStar:
+            star_pos = count
+
             variable_name = seq_pattern.name
 
             if variable_name is not None:
                 assert "." not in variable_name, variable_name
                 assert "!" not in variable_name, variable_name
 
-                star_pos = count
-
                 # Last one
                 if star_pos == len(pattern.patterns):
                     slice_value = slice(count)
@@ -389,9 +389,14 @@ def _buildMatchOr(provider, pattern, make_against, source_ref):
             source_ref=source_ref,
         )
 
-        or_condition_list.append(
-            makeAndNode(values=or_conditions, source_ref=source_ref)
-        )
+        if or_conditions:
+            or_condition_list.append(
+                makeAndNode(values=or_conditions, source_ref=source_ref)
+            )
+        else:
+            or_condition_list.append(
+                makeConstantRefNode(constant=True, source_ref=source_ref)
+            )
 
         # TODO: Inconsistency somewhere, can return empty.
         or_assignments_list.append(or_assignments or None)
@@ -439,7 +444,7 @@ def _buildMatchOr(provider, pattern, make_against, source_ref):
             provider=provider, name="match_or", source_ref=source_ref
         )
 
-        outline_body.setChild("body", body)
+        outline_body.setChildBody(body)
 
         conditions = (outline_body,)
         assignments = None
