@@ -22,7 +22,6 @@ from __future__ import print_function
 # nuitka-skip-unless-imports: numpy
 
 # nuitka-project: --standalone
-# nuitka-project: --enable-plugin=numpy
 
 # Make sure, the usual bad ones are not included with anti-bloat.
 
@@ -38,7 +37,11 @@ a = numpy.arange(15).reshape(3, 5)
 
 print("An array", a)
 
-import numpy.random._bounded_integers
+try:
+    import numpy.random._bounded_integers
+except ImportError:
+    print("didn't load numpy.random._bounded_integers")
+    pass
 
 import numpy.lib.recfunctions
 
